@@ -1,31 +1,33 @@
-use std::{
-    io::{Cursor, Read},
-    str::FromStr,
-    usize,
-};
+use core::num;
+use std::{any, fmt::format, io::{BufRead, Cursor, Read, Write}, str::FromStr, usize};
 
 use crate::utils::{from_hex, to_hex};
 use anyhow::*;
 use byteorder::{LittleEndian, ReadBytesExt};
 use num_traits::{FromPrimitive, ToPrimitive};
 use serde::*;
-use thiserror::*;
+use snafu::*;
 use wasm_bindgen::{prelude::*, throw_str};
 
 use crate::OpCodes;
 
-#[derive(Debug, Error)]
+#[derive(Debug, Snafu)]
 pub enum ScriptErrors {
-    #[error("Error deserialising Script: {}", error)]
+    #[snafu(display("Error deserialising Script: {}", error))]
     Deserialise { error: anyhow::Error },
 
-    #[error("Error serialising Script field {}: {}", reason, error)]
-    Serialise { reason: String, error: anyhow::Error },
+    #[snafu(display("Error serialising Script field {}: {}", reason, error))]
+    Serialise {
+        reason: String,
+        error: anyhow::Error,
+    },
 }
 
 #[wasm_bindgen]
 #[derive(Debug, Default, Clone, PartialEq, Eq, Serialize, Deserialize)]
-pub struct Script(#[serde(serialize_with = "to_hex", deserialize_with = "from_hex")] pub(crate) Vec<u8>);
+pub struct Script(
+    #[serde(serialize_with = "to_hex", deserialize_with = "from_hex")] pub(crate) Vec<u8>,
+);
 
 /**
  * Serialise Methods
@@ -38,7 +40,12 @@ impl Script {
         Ok(self.read_opcodes(&mut cursor, String::new(), extended)?)
     }
 
-    fn read_opcodes(&self, cursor: &mut Cursor<Vec<u8>>, builder_str: String, extended: bool) -> Result<String, ScriptErrors> {
+    fn read_opcodes(
+        &self,
+        cursor: &mut Cursor<Vec<u8>>,
+        builder_str: String,
+        extended: bool,
+    ) -> Result<String, ScriptErrors> {
         if cursor.position() >= self.0.len() as u64 {
             return Ok(builder_str);
         }
@@ -64,10 +71,10 @@ impl Script {
         }
 
         let opcode_str = match FromPrimitive::from_u8(byte) {
-            Some(v @ OpCodes::OP_0) => match extended {
-                true => v.to_string(),
-                false => 0.to_string(),
-            },
+            // Some(v @ OpCodes::OP_0) => match extended {
+            //     true => v.to_string(),
+            //     false => 0.to_string(),
+            // },
             Some(v @ OpCodes::OP_PUSHDATA1) => Script::format_pushdata_string(cursor, v, extended)?,
             Some(v @ OpCodes::OP_PUSHDATA2) => Script::format_pushdata_string(cursor, v, extended)?,
             Some(v @ OpCodes::OP_PUSHDATA4) => Script::format_pushdata_string(cursor, v, extended)?,
@@ -84,11 +91,18 @@ impl Script {
         Script::read_opcodes(&self, cursor, new_str, extended)
     }
 
-    fn get_pushdata_length(cursor: &mut Cursor<Vec<u8>>, opcode: OpCodes) -> Result<usize, ScriptErrors> {
+    fn get_pushdata_length(
+        cursor: &mut Cursor<Vec<u8>>,
+        opcode: OpCodes,
+    ) -> Result<usize, ScriptErrors> {
         let result = match opcode {
             OpCodes::OP_PUSHDATA1 => cursor.read_u8().and_then(|x| Ok(x as usize)),
-            OpCodes::OP_PUSHDATA2 => cursor.read_u16::<LittleEndian>().and_then(|x| Ok(x as usize)),
-            OpCodes::OP_PUSHDATA4 => cursor.read_u32::<LittleEndian>().and_then(|x| Ok(x as usize)),
+            OpCodes::OP_PUSHDATA2 => cursor
+                .read_u16::<LittleEndian>()
+                .and_then(|x| Ok(x as usize)),
+            OpCodes::OP_PUSHDATA4 => cursor
+                .read_u32::<LittleEndian>()
+                .and_then(|x| Ok(x as usize)),
             _ => {
                 return Err(ScriptErrors::Serialise {
                     reason: format!("Given opcode {} is not pushdata", opcode),
@@ -116,11 +130,12 @@ impl Script {
         }
     }
 
-    /**
-     * OpCodes such as OP_PUSH or the numerical OpCodes (OP_1-OP_16)
-     */
-    fn get_special_opcode(byte: u8, extended: bool, cursor: &mut Cursor<Vec<u8>>) -> Result<Option<String>, ScriptErrors> {
-        let code = match byte {
+    fn get_special_opcode(
+        byte: u8,
+        extended: bool,
+        cursor: &mut Cursor<Vec<u8>>,
+    ) -> Result<Option<String>, ScriptErrors> {
+        Ok(match byte {
             size @ 0x01..=0x4b => {
                 let pushdata = Script::get_pushdata(cursor, size as usize)?;
                 match extended {
@@ -128,17 +143,15 @@ impl Script {
                     false => Some(pushdata),
                 }
             }
-
-            v @ 82..=96 => match OpCodes::from_u8(v) {
-                Some(num_opcode) => Some(num_opcode.to_string()),
-                None => None,
-            },
             _ => None,
-        };
-        Ok(code)
+        })
     }
 
-    fn format_pushdata_string(cursor: &mut Cursor<Vec<u8>>, v: OpCodes, extended: bool) -> Result<String, ScriptErrors> {
+    fn format_pushdata_string(
+        cursor: &mut Cursor<Vec<u8>>,
+        v: OpCodes,
+        extended: bool,
+    ) -> Result<String, ScriptErrors> {
         let size = Script::get_pushdata_length(cursor, v)?;
         let pushdata = Script::get_pushdata(cursor, size)?;
         Ok(match extended {
@@ -160,94 +173,82 @@ impl Script {
     }
 
     pub(crate) fn from_asm_string_impl(asm: String) -> Result<Script, ScriptErrors> {
-        let mut chunks = asm.split(" ").into_iter();
-        let mut buffer: Vec<u8> = Vec::new();
+      let mut chunks = asm.split(" ").into_iter();
+      let mut buffer: Vec<u8> = Vec::new();
 
-        while let Some(code) = chunks.next() {
-            // Number OP_CODES
-            if let Ok(num_code) = u8::from_str(code) {
-                match num_code {
-                    v @ 0 => buffer.push(v),
-                    v @ 1..=16 => buffer.push(v + 80),
-                    _ => (),
-                }
-
-                continue;
-            }
-
-            // Standard OP_CODES
-            if let Ok(opcode) = OpCodes::from_str(code) {
-                if let Some(opcode_byte) = opcode.to_u8() {
-                    buffer.push(opcode_byte);
-                }
-                continue;
-            }
-
-            // PUSHDATA OP_CODES
-            let length = code.len() / 2;
-            match length {
-                op_push @ 0x01..=0x4b => {
-                    buffer.push(op_push as u8);
-                    match hex::decode(code) {
-                        Ok(v) => buffer.append(&mut v.clone()),
-                        Err(e) => return Err(ScriptErrors::Deserialise { error: anyhow!(e) }),
-                    }
-                }
-                op_pushdata1_size @ 0x4c..=0xFF => {
-                    match OpCodes::OP_PUSHDATA1.to_u8() {
-                        Some(pushdata1_byte) => buffer.push(pushdata1_byte),
-                        None => {
-                            return Err(ScriptErrors::Deserialise {
-                                error: anyhow!("Unable to deserialise OP_PUSHDATA1 Code to u8"),
-                            })
-                        }
-                    };
-
-                    buffer.push(op_pushdata1_size as u8);
-                    match hex::decode(code) {
-                        Ok(v) => buffer.append(&mut v.clone()),
-                        Err(e) => return Err(ScriptErrors::Deserialise { error: anyhow!(e) }),
-                    }
-                }
-                op_pushdata2_size @ 0x100..=0xFFFF => {
-                    match OpCodes::OP_PUSHDATA2.to_u8() {
-                        Some(pushdata2_byte) => buffer.push(pushdata2_byte),
-                        None => {
-                            return Err(ScriptErrors::Deserialise {
-                                error: anyhow!("Unable to deserialise OP_PUSHDATA2 Code to u8"),
-                            })
-                        }
-                    };
-
-                    buffer.push(op_pushdata2_size as u8);
-                    match hex::decode(code) {
-                        Ok(v) => buffer.append(&mut v.clone()),
-                        Err(e) => return Err(ScriptErrors::Deserialise { error: anyhow!(e) }),
-                    }
-                }
-                size => {
-                    // Cant do a standard match because 0xFFFFFFFF is too large
-                    if size > 0x10000 && size <= 0xFFFFFFFF {
-                        match OpCodes::OP_PUSHDATA4.to_u8() {
-                            Some(pushdata4_byte) => buffer.push(pushdata4_byte),
-                            None => {
-                                return Err(ScriptErrors::Deserialise {
-                                    error: anyhow!("Unable to deserialise OP_PUSHDATA4 Code to u8"),
-                                })
-                            }
-                        };
-
-                        buffer.push(size as u8);
-                        match hex::decode(code) {
-                            Ok(v) => buffer.append(&mut v.clone()),
-                            Err(e) => return Err(ScriptErrors::Deserialise { error: anyhow!(e) }),
-                        }
-                    }
-                }
-            }
+      while let Some(code) = chunks.next() {
+        // Number OP_CODES
+        if let Ok(num_code) = u8::from_str(code) {
+          match num_code {
+            v @ 0 => buffer.push(v),
+            v @ 1..=16 => buffer.push(v + 80),
+            _ => ()
+          }
+          
+          continue;
         }
 
-        Ok(Script(buffer))
+        // Standard OP_CODES
+        if let Ok(opcode) = OpCodes::from_str(code) {
+          if let Some(opcode_byte) = opcode.to_u8() {
+            buffer.push(opcode_byte);
+          }
+          continue;
+        }
+
+        // PUSHDATA OP_CODES
+        let length = code.len() / 2;
+        match length {
+          op_push @ 0x01..=0x4b => {
+            buffer.push(op_push as u8);
+            match hex::decode(code) {
+              Ok(v) => buffer.append(&mut v.clone()),
+              Err(e) => return Err(ScriptErrors::Deserialise{ error: anyhow!(e) })
+            }
+          },
+          op_pushdata1_size @ 0x4c..=0xFF => {
+            match OpCodes::OP_PUSHDATA1.to_u8() {
+              Some(pushdata1_byte) => buffer.push(pushdata1_byte),
+              None => return Err(ScriptErrors::Deserialise{ error: anyhow!("Unable to deserialise OP_PUSHDATA1 Code to u8") })
+            };
+
+            buffer.push(op_pushdata1_size as u8);
+            match hex::decode(code) {
+              Ok(v) => buffer.append(&mut v.clone()),
+              Err(e) => return Err(ScriptErrors::Deserialise{ error: anyhow!(e) })
+            }
+          },
+          op_pushdata2_size @ 0x100..=0xFFFF => {
+            match OpCodes::OP_PUSHDATA2.to_u8() {
+              Some(pushdata2_byte) => buffer.push(pushdata2_byte),
+              None => return Err(ScriptErrors::Deserialise{ error: anyhow!("Unable to deserialise OP_PUSHDATA2 Code to u8") })
+            };
+
+            buffer.push(op_pushdata2_size as u8);
+            match hex::decode(code) {
+              Ok(v) => buffer.append(&mut v.clone()),
+              Err(e) => return Err(ScriptErrors::Deserialise{ error: anyhow!(e) })
+            }
+          },
+          size => {
+            // Cant do a standard match because 0xFFFFFFFF is too large
+            if size > 0x10000 && size <= 0xFFFFFFFF {
+              match OpCodes::OP_PUSHDATA4.to_u8() {
+                Some(pushdata4_byte) => buffer.push(pushdata4_byte),
+                None => return Err(ScriptErrors::Deserialise{ error: anyhow!("Unable to deserialise OP_PUSHDATA4 Code to u8") })
+              };
+  
+              buffer.push(size as u8);
+              match hex::decode(code) {
+                Ok(v) => buffer.append(&mut v.clone()),
+                Err(e) => return Err(ScriptErrors::Deserialise{ error: anyhow!(e) })
+              }
+            }
+          }
+        }
+      }
+
+      Ok(Script(buffer))
     }
 }
 
@@ -256,24 +257,17 @@ impl Script {
  */
 #[wasm_bindgen]
 impl Script {
-    #[wasm_bindgen(js_name = toBytes)]
     pub fn to_bytes(&self) -> Vec<u8> {
         self.0.clone()
     }
 
-    #[wasm_bindgen(js_name = fromBytes)]
     pub fn from_bytes(bytes: Vec<u8>) -> Script {
         Script(bytes)
     }
 
-    #[wasm_bindgen(js_name = toHex)]
     pub fn to_hex(&self) -> String {
         hex::encode(self.to_bytes())
     }
-
-    pub fn remove_codeseparators(&mut self) {
-        self.0 = self.0.clone().into_iter().filter(|x| *x != OpCodes::OP_CODESEPARATOR.to_u8().unwrap()).collect();
-    }
 }
 
 /**
